// Wiloke Resizable
(function(factory) {
    if (typeof define === 'function' && define.amd) {
        // AMD
        define(['jquery'], factory);
    } else if (typeof exports === 'object') {
        // CommonJS
        factory(require('jquery'));
    } else {
        // Browser globals
        factory(jQuery);
    }
}(function($) {

    'use strict';

    var WilokeResizable = function(el, options) {
        this.$el = el;
        this.options = $.extend({}, WilokeResizable.defaults, options);
        this.handle = this.$el.find(this.options.handleSelector);
        this.resizeHeight = this.options.resizeHeight;
        this.resizeHeightFrom = this.options.resizeHeightFrom;
        this.resizeWidth = this.options.resizeWidth;
        this.resizeWidthFrom = this.options.resizeWidthFrom;
        this.size = 0;
        this.column = 1;
        this.width = 1;
        this.height = 1;
        this.startPos = {};
        this.init();
    }

    WilokeResizable.defaults =  {
        // selector for handle that starts dragging
        handleSelector: null,
        // resize the width
        resizeWidth: true,
        // resize the height
        resizeHeight: true,            
        // the side that the width resizing is relative to
        resizeWidthFrom: 'right',
        // the side that the height resizing is relative to
        resizeHeightFrom: 'bottom',            
        // hook into start drag operation (event passed)
        onDragStart: null,
        // hook into stop drag operation (event passed)
        onDragEnd: null,
        // hook into each drag operation (event passed)
        onDrag: null,
        // disable touch-action on $handle
        // prevents browser level actions like forward back gestures
        touchActionNone: true
    };

    WilokeResizable.prototype = {

        init: function() {

            var _this = this;

            this.$el.addClass("resizable");

            this.handle.bind('mousedown.rsz', $.proxy(this.startDragging, this));
        },

        startDragging: function(e) {
            
            var self = $(e.currentTarget);

            if( self.hasClass('right') ) {
                this.resizeHeight = false;
                this.resizeWidth = true;
                this.resizeWidthFrom = "right";
            }

            if( self.hasClass('bottom') ) {
                this.resizeWidth = false;
                this.resizeHeight = true;
                this.resizeHeightFrom = "bottom";
            }

            this.$el.css({
                'transition': 'none',
                'z-index': 2,
            }).addClass('resizing');

            this.handle.data('is-dragg', true);

            this.size = parseInt(this.$el.attr('data-size'));
            this.column = parseInt(this.$el.attr('data-column'));
            this.width = parseInt(this.$el.attr('data-width'));
            this.height = parseInt(this.$el.attr('data-height'));
            this.startPos = this.getMousePos(e);

            this.startPos.width = parseInt(this.$el.innerWidth(), 10);
            this.startPos.height = parseInt(this.$el.innerHeight(), 10);
            
            $(document).bind('mousemove.rsz', $.proxy(this.onDrag, this));
            $(document).bind('mouseup.rsz', $.proxy(this.stopDragging, this));  
            $(document).bind('selectstart.rsz', this.noop);

        },

        noop: function(e) {
            e.stopPropagation();
            e.preventDefault();
        },

        onDrag: function(e) {
            
            var pos = this.getMousePos(e), newWidth, newHeight;

            if (this.resizeWidthFrom === 'right') {

                if(pos.x > this.startPos.x && pos.x - this.startPos.x > this.size/2) {
                    newWidth = this.startPos.width + this.size;
                    this.$el.data('status', 'no-change').attr('data-width', this.width + 1);
                } else if(this.startPos.x > pos.x && this.startPos.x - pos.x > this.size/2) {
                    newWidth = this.startPos.width - this.size;
                    this.$el.data('status', 'no-change').attr('data-width', this.width - 1);
                } else {
                    newWidth = this.startPos.width + pos.x - this.startPos.x;
                    this.$el.data('status', 'change');
                }

                if(newWidth > this.size * this.column) {
                    newWidth = this.size * this.column;
                    this.$el.attr('data-width', this.column); 
                } else if(newWidth < this.size) {
                    newWidth = this.size;
                    this.$el.attr('data-width', 1); 
                }

            } 

            if (this.resizeHeightFrom === 'bottom') {

                if(pos.y > this.startPos.y && pos.y - this.startPos.y > this.size/2) {
                    newHeight = this.startPos.height + this.size;
                    this.$el.data('status', 'no-change').attr('data-height', this.height + 1);
                } else if(this.startPos.y > pos.y && this.startPos.y - pos.y > this.size/2) {
                    newHeight = this.startPos.height - this.size;
                    this.$el.data('status', 'no-change').attr('data-height', this.height - 1);
                } else {
                    newHeight = this.startPos.height + pos.y - this.startPos.y;
                    this.$el.data('status', 'change');
                }

                if(newHeight > this.size * this.column) {
                    newHeight = this.size * this.column;
                    this.$el.attr('data-height', this.column);  
                } else if(newHeight < this.size) {
                    newHeight = this.size;
                    this.$el.attr('data-height', 1);  
                }

            }

            if( this.handle.data('is-dragg') ) {

                if (this.resizeHeight) {
                    this.$el.innerHeight(newHeight);                    
                }

                if (this.resizeWidth) {
                    this.$el.innerWidth(newWidth); 
                }

                if (typeof this.options.onDrag === "function") {
                    this.options.onDrag.apply(this);
                }
            }
        },

        stopDragging: function(e) {
            e.stopPropagation();
            e.preventDefault();

            var pos = this.getMousePos(e), newWidth, newHeight;

            if( this.$el.data('status') == 'change' ) {

                if (this.resizeWidthFrom === 'right') {

                    if( pos.x > this.startPos.x ) {

                        if( pos.x - this.startPos.x > this.size/2) {
                            newWidth = this.startPos.width  + this.size;
                        } else {
                            newWidth = this.startPos.width;
                        }

                    } else {

                        if(this.startPos.x - pos.x > this.size/2) {
                            newWidth = this.startPos.width  - this.size;
                        } else {
                            newWidth = this.startPos.width;
                        }

                    }

                    if(newWidth > this.size * this.column) {
                        newWidth = this.size * this.column;
                        this.$el.attr('data-width', this.column); 
                    } else if(newWidth < this.size) {
                        newWidth = this.size;
                        this.$el.attr('data-width', 1); 
                    }
                }

                if( this.resizeHeightFrom === 'bottom' ) {

                    if( pos.y > this.startPos.y ) {

                        if( pos.y - this.startPos.y > this.size/2) {
                            newHeight = this.startPos.height  + this.size;
                        } else {
                            newHeight = this.startPos.height;
                        }

                    } else {

                        if(this.startPos.y - pos.y > this.size/2) {
                            newHeight = this.startPos.height  - this.size;
                        } else {
                            newHeight = this.startPos.height;
                        }

                    }

                    if(newHeight > this.size * this.column) {
                        newHeight = this.size * this.column;
                        this.$el.attr('data-height', this.column);  
                    } else if(newHeight < this.size) {
                        newHeight = this.size;
                        this.$el.attr('data-height', 1);  
                    }
                }

                if (this.resizeWidth) {
                    this.$el.innerWidth(newWidth); 
                }

                if (this.resizeHeight) {
                    this.$el.innerHeight(newHeight); 
                }
            }

            this.$el.css({
                'transition': '',
                'z-index': '',
            }).removeClass('resizing');

            
            $(document).unbind('mousemove.rsz', this.onDrag);
            $(document).unbind('mouseup.rsz', this.stopDragging);
            $(document).unbind('selectstart.rsz', this.noop);

            this.handle.data('is-dragg', false);

            if (typeof this.options.onDragEnd === "function") {

                var index = this.$el.attr('data-order'),
                    sizeWidth = this.$el.attr('data-width'),
                    sizeHeight = this.$el.attr('data-height');

                this.options.onDragEnd.apply(this, [index, sizeWidth, sizeHeight]);
            }

            this.size = 0;
            this.column = 1;
            this.width = 1;
            this.height = 1;
            this.startPos = {};

            return false;
        },

        getMousePos: function(e) {

            var pos = { x: 0, y: 0, width: 0, height: 0 };

            if (typeof e.clientX === "number") {
                pos.x = e.clientX;
                pos.y = e.clientY;
            } else if (e.originalEvent.touches) {
                pos.x = e.originalEvent.touches[0].clientX;
                pos.y = e.originalEvent.touches[0].clientY;
            } else
                return null;

            return pos;
        }
    }

    window.WilokeResizable = WilokeResizable;

    $.fn.WilokeResizable = function(options) {

        return this.each(function(index, el) {
            var self = $(this),
                data = self.data('WilokeResizable');
                
            if(!data) {
                data = new WilokeResizable($(this), options);
                self.data('WilokeResizable', data);
            }
        });
    }

}));